import React, { useRef } from 'react';
import styled from "styled-components";
import Loading from '../../util/Loading';
import Button from "../../util/Button";
import PlayerScoreTile from "./PlayerScoreTile";
import useGameInfo from '../../../hooks/useGameInfo';
import ClickToCopy from '../../util/ClickToCopy';
import { dispatch } from '../../../domain/gameStore';
import { linkGame } from '../../../domain/apiMethods';
import GameCreationForm from '../../util/GameCreationForm';
import useMap from '../../../hooks/useMap';
import useMarkersFromGuesses from '../../../hooks/useMarkersFromGuesses';
import { useState } from 'react';

const Container = styled.div`
  display: flex;
  flex-flow: column nowrap;
  justify-content: space-between;
  align-items: center;
  height: 100%;
`

const ScoreBoardContainer = styled.div`
  flex: 2;
  display: flex;
  flex-flow: row wrap;
  justify-content: space-evenly;
  align-items: flex-start;
  align-content: space-around;
  max-width: 80%;
  margin-top: 2em;
`

const LinkedGameContainer = styled.div`
  display: flex;
  justify-content: center;
  width: 100%;
  margin-top: 1em;
  margin-bottom: 10em;
`

const Label = styled.span`
  padding: 0.2em;
  margin-bottom: 0.5em;
`

const StyledButton = styled(Button)`
  margin-top: 0.5em;
`

const MapDiv = styled.div`
  position: absolute;
  height: 30%;
  width: 90%;
`;

const MapContainer = styled.div`
  height: 100%;
  display: flex;
  justify-content: center;
`;

const getUrl = gameId => {
  const u = new URL(window.location.href);
  u.searchParams.append("summary", gameId);
  return u.href;
}

const SummaryMap = ({ mapDivRef, mapRef, players, coords, roundNum }) => {
  // get just the relevant players to put on the map
  const playersFiltered = players
    ?.filter(({ guesses }) => guesses[roundNum] && guesses[roundNum].score !== null)
    ?.map(({ name, totalScore, guesses }) => ({ name, totalScore, guess: guesses[roundNum] }));
  const targetPoint = coords?.[roundNum] ?? null;

  useMarkersFromGuesses(mapRef, playersFiltered, targetPoint);

  return (
    <MapContainer>
      <MapDiv ref={mapDivRef}/>
    </MapContainer>
  );
}

const ScoreBoard = ({ players }) => (
  <ScoreBoardContainer>
    {
      players
        .filter(({ currentRound }) => currentRound === null)
        .sort((p1, p2) => p1.totalScore > p2.totalScore ? -1 : (p1.totalScore < p2.totalScore ? 1 : 0))
        .map((data) => <PlayerScoreTile key={data.name} {...data} />)
    }
  </ScoreBoardContainer>
)

const LinkedGame = React.memo(({ linkedGame, gameId }) => (
  <LinkedGameContainer>
    {
      linkedGame
        ? <StyledButton onClick={() => dispatch.goToLobby(linkedGame)}>
            Continue to Linked Game Lobby
          </StyledButton>
        : <GameCreationForm afterCreate={linkId => linkGame(gameId, linkId)}/>
    }
  </LinkedGameContainer>
));

export default () => {
  // create the map
  // const mapDivRef = useRef(null);
  // const mapRef = useMap(mapDivRef, 0, 0, 1);

  // poll the game state
  const { gameId, players, coords, linkedGame }  = useGameInfo();

  // set up round number selection
  // const [roundNum, setRoundNum] = useState("0");

  if (!players) {
    return <Loading/>
  }

  return (
    <Container>
      {/* <SummaryMap {...{ mapDivRef, mapRef, players, coords, roundNum }}/> */}
      <ScoreBoard {...{ players }}/>
      <LinkedGame {...{ linkedGame, gameId }} />
      <Label><ClickToCopy text={getUrl(gameId)}>Click here to copy a link to this summary!</ClickToCopy></Label>
    </Container>
  );
}
